from __future__ import unicode_literals

from django.apps import AppConfig


class UserExtensionsConfig(AppConfig):
    name = 'userextensions'
    verbose_name = "User Extensions"

    def ready(self):
        import userextensions.signals



from django import template

# import models
from userextensions.models import UserFavorite, UserPreference

register = template.Library()


@register.simple_tag(name='is_favorite')
def is_favorite(request):
    try:
        user = request.user
        path = getattr(request, 'get_full_path')()
        favorite = UserFavorite.objects.get_object_or_none(url__icontains=path, user=user)
        if favorite:
            return favorite.id
        else:
            return False
    except:
        return None


@register.simple_tag(name='get_theme')
def get_theme(request):
    try:
        user = request.user
        preference = UserPreference.objects.get_object_or_none(user=user)
        if preference:
            return preference.theme
        else:
            return None
    except:
        return None



"""
This file contains views that perform a well defined action and redirect to a rendered page, typically the referrer. No
page rendering views are contained here.
"""

from django.shortcuts import redirect
from django.contrib import messages
from django.conf import settings
from django.utils import timezone
from django.views.generic import View, DeleteView
from rest_framework.authtoken.models import Token
from braces.views import LoginRequiredMixin

# import models
from django.contrib.auth.models import User, Group
from userextensions.models import (UserRecent, UserFavorite, ServiceAccount, ServiceAccountTokenHistory)


class RefreshApiToken(LoginRequiredMixin, View):
    """ delete current user API (auth) token and create a new one """
    def post(self, request):
        try:
            token = Token.objects.get(user=request.user)
            token.delete()
            Token.objects.get_or_create(user=request.user)
        except Exception as err:
            messages.add_message(request, messages.ERROR, f'Could not complete requested action: {err}',
                                 extra_tags='alert-danger')
        return redirect(self.request.META.get('HTTP_REFERER'))


class RefreshSrvAcctApiToken(LoginRequiredMixin, View):
    """ delete current the API (auth) token for a provided service account and create a new one """
    def post(self, request):
        srv_acct_id = self.request.GET.dict().get('srv_acct_id', None)
        try:
            srv_acct = ServiceAccount.objects.get(id=srv_acct_id)
            token = Token.objects.get(user=srv_acct.user)
            token.delete()
            Token.objects.get_or_create(user=srv_acct.user)

            # track the token refresh in ServiceAccountTokenHistory
            ServiceAccountTokenHistory.objects.create(actor=request.user, srv_acct=srv_acct)

            messages.add_message(request, messages.INFO, 'API token refreshed', extra_tags='alert-info')
        except Exception as err:
            messages.add_message(request, messages.ERROR, f'Could not complete requested action: {err}',
                                 extra_tags='alert-danger')
        return redirect(self.request.META.get('HTTP_REFERER'))


class AddFavorite(LoginRequiredMixin, View):
    """ add (the current) url to the list of user favorites """
    def post(self, request):
        referrer = self.request.META.get('HTTP_REFERER')
        name = self.request.POST.dict().get('name', None)

        # do not track url if user is not authenticated or user can not be determined
        if not request.user.is_authenticated:
            return redirect(referrer)
        user = getattr(request, 'user')
        if not user:
            return redirect(referrer)

        try:
            recent, is_new = UserFavorite.objects.get_or_create(url=referrer,
                                                                user=request.user,
                                                                defaults=dict(url=referrer,
                                                                              user=request.user,
                                                                              name=name,
                                                                              updated_at=timezone.now())
                                                                )
            if is_new:
                messages.add_message(request, messages.INFO, 'Successfully added favorite',
                                     extra_tags='alert-info')
            else:
                messages.add_message(request, messages.INFO, 'This url is already in your favorites',
                                     extra_tags='alert-info')
        except Exception as err:
            messages.add_message(request, messages.ERROR, f'Error adding favorite: {err}',
                                 extra_tags='alert-danger')
        return redirect(referrer)


class DeleteFavorite(LoginRequiredMixin, DeleteView):
    """ delete a favorite (by pk) and return to the referring page """
    def delete(self, request, *args, **kwargs):
        UserFavorite.objects.filter(**kwargs).delete()
        messages.add_message(self.request, messages.INFO, 'Favorite successfully deleted', extra_tags='alert-info')
        return redirect(self.request.META.get('HTTP_REFERER'))


class DeleteRecent(LoginRequiredMixin, DeleteView):
    """ delete a recent (by pk) and return to the referring page """
    def delete(self, request, *args, **kwargs):
        UserRecent.objects.filter(**kwargs).delete()
        messages.add_message(self.request, messages.INFO, 'Recent successfully deleted', extra_tags='alert-info')
        return redirect(self.request.META.get('HTTP_REFERER'))


class UserLoginRedirect(LoginRequiredMixin, View):
    """ Check if a user has a preferred 'start page' to load after login. If so, redirect to that page after login, else
        redirect to the project root page.
        To enable this redirect, set the LOGIN_REDIRECT_URL parameter in the settings.py to
        /userextensions/user_login_redirect and include userextensions.urls in the project level urls.py
    """
    @staticmethod
    def get(request):
        # get the users preferred start page
        try:
            start_page = request.user.preference.start_page
            if start_page:
                return redirect(start_page)
            else:
                return redirect(getattr(settings, 'LOGIN_REDIRECT_URL_DEFAULT', '/'))
        except Exception as err:
            messages.add_message(request, messages.ERROR, f'Error getting start page; redirect to root: {err}',
                                 extra_tags='alert-danger')
            return redirect("/")


class SetStartPage(LoginRequiredMixin, View):
    """ set the current page as the users preferred 'start page' to be redirected to after login """
    def post(self, request):
        referrer = self.request.META.get('HTTP_REFERER')
        try:
            # do not set start page if user is not authenticated or user can not be determined
            if not request.user.is_authenticated:
                return redirect(referrer)

            # set current url as start page
            request.user.preference.start_page = referrer
            request.user.preference.save()
            messages.add_message(request, messages.INFO, 'start page updated!', extra_tags='alert-info')
        except Exception as err:
            messages.add_message(request, messages.ERROR, f'Error setting start page: {err}',
                                 extra_tags='alert-danger')
        return redirect(referrer)


class CreateServiceAccount(LoginRequiredMixin, View):
    """ create a new service account based on a provided group """
    def post(self, request):
        referrer = self.request.META.get('HTTP_REFERER')
        try:
            group_id = self.request.GET.dict().get('group_id', None)
            description = self.request.GET.dict().get('description', None)
            group = Group.objects.get(id=group_id)

            # only create service account if user is a member of the linked group
            if request.user not in group.user_set.all():
                messages.add_message(request, messages.ERROR, 'You are not authorized to delete this service account',
                                     extra_tags='alert-danger')
                return redirect(referrer)

            srv_acct = ServiceAccount.objects.create(group=group, description=description)
            messages.add_message(request, messages.INFO, f'Service account {srv_acct.user.username} created',
                                 extra_tags='alert-info')
        except Exception as err:
            messages.add_message(request, messages.ERROR, f'Error creating service account: {err}',
                                 extra_tags='alert-danger')
        return redirect(referrer)


class DeleteServiceAccount(LoginRequiredMixin, View):
    """ delete a service account """
    def post(self, request):
        referrer = self.request.META.get('HTTP_REFERER')
        try:
            srv_acct_id = self.request.GET.dict().get('srv_acct_id', None)
            srv_acct = ServiceAccount.objects.get_object_or_none(id=srv_acct_id)
            srv_acct_name = srv_acct.user.username
            if not srv_acct:
                messages.add_message(request, messages.ERROR, 'invalid service account provided',
                                     extra_tags='alert-danger')
                return redirect(referrer)

            # only delete if user is a member of the group linked to this srv_acct
            if request.user not in srv_acct.group.user_set.all():
                messages.add_message(request, messages.ERROR, 'You are not authorized to delete this service account',
                                     extra_tags='alert-danger')
                return redirect(referrer)

            # delete the User entry; all cascaded models (including ServiceAccount) will be deleted
            srv_acct.user.delete()
            messages.add_message(request, messages.INFO, f'Service account {srv_acct_name} deleted',
                                 extra_tags='alert-info')

        except Exception as err:
            messages.add_message(request, messages.ERROR, f'Error deleting service account: {err}',
                                 extra_tags='alert-danger')
        return redirect(referrer)


class EnableServiceAccount(LoginRequiredMixin, View):
    """ enable a service account """
    def post(self, request):
        referrer = self.request.META.get('HTTP_REFERER')
        try:
            srv_acct_id = self.request.GET.dict().get('srv_acct_id', None)
            srv_acct = ServiceAccount.objects.get_object_or_none(id=srv_acct_id)
            if not srv_acct:
                messages.add_message(request, messages.ERROR, 'invalid service account provided',
                                     extra_tags='alert-danger')
                return redirect(referrer)

            # only enable if user is a member of the group linked to this srv_acct
            if request.user not in srv_acct.group.user_set.all():
                messages.add_message(request, messages.ERROR, 'You are not authorized to enable this service account',
                                     extra_tags='alert-danger')
                return redirect(referrer)

            srv_acct.enabled = True
            srv_acct.save()
            messages.add_message(request, messages.INFO, f'Service account {srv_acct.user.username} enabled',
                                 extra_tags='alert-info')

        except Exception as err:
            messages.add_message(request, messages.ERROR, f'Error enabling service account: {err}',
                                 extra_tags='alert-danger')
        return redirect(referrer)


class DisableServiceAccount(LoginRequiredMixin, View):
    """ disable a service account """
    def post(self, request):
        referrer = self.request.META.get('HTTP_REFERER')
        try:
            srv_acct_id = self.request.GET.dict().get('srv_acct_id', None)
            srv_acct = ServiceAccount.objects.get_object_or_none(id=srv_acct_id)
            if not srv_acct:
                messages.add_message(request, messages.ERROR, 'invalid service account provided',
                                     extra_tags='alert-danger')
                return redirect(referrer)

            # only enable if user is a member of the group linked to this srv_acct
            if request.user not in srv_acct.group.user_set.all():
                messages.add_message(request, messages.ERROR, 'You are not authorized to disable this service account',
                                     extra_tags='alert-danger')
                return redirect(referrer)
            srv_acct.enabled = False
            srv_acct.save()
            messages.add_message(request, messages.INFO, f'Service account {srv_acct.user.username} disabled',
                                 extra_tags='alert-info')

        except Exception as err:
            messages.add_message(request, messages.ERROR, f'Error disabling service account: {err}',
                                 extra_tags='alert-danger')
        return redirect(referrer)


class AdminEnableServiceAccount(LoginRequiredMixin, View):
    """ admin enable a service account """
    def post(self, request):
        referrer = self.request.META.get('HTTP_REFERER')
        try:
            srv_acct_id = self.request.GET.dict().get('srv_acct_id', None)
            srv_acct = ServiceAccount.objects.get_object_or_none(id=srv_acct_id)
            if not srv_acct:
                messages.add_message(request, messages.ERROR, 'invalid service account provided',
                                     extra_tags='alert-danger')
                return redirect(referrer)

            # only enable if user is an admin
            if not request.user.is_superuser:
                messages.add_message(request, messages.ERROR, 'You must be an admin to enable this service account',
                                     extra_tags='alert-danger')
                return redirect(referrer)

            srv_acct.admin_enabled = True
            srv_acct.save()
            messages.add_message(request, messages.INFO, f'Service account {srv_acct.user.username} is admin enabled',
                                 extra_tags='alert-info')

        except Exception as err:
            messages.add_message(request, messages.ERROR, f'Error admin-enabling service account: {err}',
                                 extra_tags='alert-danger')
        return redirect(referrer)


class AdminDisableServiceAccount(LoginRequiredMixin, View):
    """ admin disable a service account """
    def post(self, request):
        referrer = self.request.META.get('HTTP_REFERER')
        try:
            srv_acct_id = self.request.GET.dict().get('srv_acct_id', None)
            srv_acct = ServiceAccount.objects.get_object_or_none(id=srv_acct_id)
            if not srv_acct:
                messages.add_message(request, messages.ERROR, 'invalid service account provided',
                                     extra_tags='alert-danger')
                return redirect(referrer)

            # only disable if user is an admin
            if not request.user.is_superuser:
                messages.add_message(request, messages.ERROR, 'You must be an admin to disable this service account',
                                     extra_tags='alert-danger')
                return redirect(referrer)
            srv_acct.admin_enabled = False
            srv_acct.save()
            messages.add_message(request, messages.INFO, f'Service account {srv_acct.user.username} is admin disabled',
                                 extra_tags='alert-info')

        except Exception as err:
            messages.add_message(request, messages.ERROR, f'Error admin-disabling service account: {err}',
                                 extra_tags='alert-danger')
        return redirect(referrer)


class EditFavorite(LoginRequiredMixin, View):
    """ edit the name of a favorite """
    def post(self, request, *args, **kwargs):
        """ process POST request """
        redirect_url = self.request.META.get('HTTP_REFERER')
        obj_id = self.request.GET.dict().get('id', None)
        name = self.request.GET.dict().get('name', None)
        try:
            favorite = UserFavorite.objects.get_object_or_none(id=obj_id)
            if request.user != favorite.user:
                messages.add_message(request, messages.ERROR, 'you are not authorized to edit this favorite',
                                     extra_tags='alert-danger')
                return redirect(redirect_url)
            favorite.name = name[:32]
            favorite.save()
            messages.add_message(request, messages.INFO, 'favorite renamed', extra_tags='alert-success')
        except Exception as err:
            messages.add_message(request, messages.ERROR, err, extra_tags='alert-danger')
        return redirect(redirect_url)



"""
This file contains views that render a specific page for the gui.
"""

from django.shortcuts import redirect, render, reverse
from django.conf import settings
from django.contrib import messages
from django.views.generic import (View, ListView)
from handyhelpers.mixins.view_mixins import FilterByQueryParamsMixin
from braces.views import LoginRequiredMixin
from rest_framework.authtoken.models import Token

# import models
from userextensions.models import (UserRecent, UserFavorite, ServiceAccount)

# import forms
from userextensions.forms import (UserPreferenceForm, ServiceAccountForm)


class ListRecents(LoginRequiredMixin, FilterByQueryParamsMixin, ListView):
    """ Displays a list of urls the user has recently visited, rendered in a paginated, searchable, sortable bootstrap
    table. This view is filterable via query parameters. Includes links to delete individual entries.
    """
    base_template = getattr(settings, 'BASE_TEMPLATE', 'handyhelpers/handyhelpers_base.htm')
    template = 'handyhelpers/generic/generic_list.html'

    def get(self, request, *args, **kwargs):
        context = dict()
        self.queryset = UserRecent.objects.filter(user=request.user).order_by('-updated_at')
        context['base_template'] = self.base_template
        context['queryset'] = self.filter_by_query_params()
        context['title'] = 'Recents'
        context['sub_title'] = request.user.username
        context['table'] = 'userextensions/table/table_recents.htm'
        return render(request, self.template, context=context)


class ListFavorites(LoginRequiredMixin, FilterByQueryParamsMixin, ListView):
    """ Displays a list of urls user has set as favorites, rendered in a paginated, searchable, sortable bootstrap
    table. This view is filterable via query parameters. Includes links to delete individual entries. """
    base_template = getattr(settings, 'BASE_TEMPLATE', 'handyhelpers/handyhelpers_base.htm')
    template = 'handyhelpers/generic/generic_list.html'

    def get(self, request, *args, **kwargs):
        context = dict()
        self.queryset = UserFavorite.objects.filter(user=request.user).order_by('-updated_at')
        context['base_template'] = self.base_template
        context['queryset'] = self.filter_by_query_params()
        context['title'] = 'Favorites'
        context['sub_title'] = request.user.username
        context['table'] = 'userextensions/table/table_favorites.htm'
        context['modals'] = 'userextensions/form/edit_hostname.htm'
        return render(request, self.template, context=context)


class DetailUser(LoginRequiredMixin, View):
    """ Displays user details, including group configuration, API token, and configuration for theme, start page, and
    recents count. Includes link to refresh API token and modal form to edit user preferences. """
    base_template = getattr(settings, 'BASE_TEMPLATE', 'handyhelpers/handyhelpers_base.htm')
    template = 'userextensions/detail/detail_user.html'

    def get(self, request):
        context = dict()
        # include user preference form
        form_data_user_preferences = dict()
        form_data_user_preferences['form'] = UserPreferenceForm(request.POST or None, instance=request.user.preference)
        form_data_user_preferences['action'] = 'Update'
        form_data_user_preferences['action_url'] = reverse('userextensions:detail_user')
        form_data_user_preferences['title'] = '<b>Update Preferences: </b><small> {} </small>'.format(request.user)
        form_data_user_preferences['modal_name'] = 'update_user_preferences'
        context['form_data_user_preferences'] = form_data_user_preferences

        context['user'] = request.user
        context['token'] = str(Token.objects.get_or_create(user=request.user)[0])
        context['groups'] = sorted([i.name for i in request.user.groups.all()])
        context['base_template'] = self.base_template
        return render(request, self.template, context=context)

    def post(self, request):
        redirect_url = request.META.get('HTTP_REFERER')
        form = UserPreferenceForm(request.POST or None, instance=request.user.preference)
        if form.is_valid():
            form.save()
            messages.add_message(request, messages.ERROR, 'Preferences updated!', extra_tags='alert-info', )
            return redirect(redirect_url)
        else:
            for error in form.errors:
                messages.add_message(request, messages.ERROR, 'Input error: {}'.format(error),
                                     extra_tags='alert-danger', )
            return self.get(request)


class ManageServiceAccounts(LoginRequiredMixin, View):
    """ Displays service accounts this user can access (service accounts that are linked to groups this owner is a
    member of). Provides mechanisms for users to create service accounts for applicable groups, refresh API tokens, and
    enable/disable service accounts """
    base_template = getattr(settings, 'BASE_TEMPLATE', 'handyhelpers/handyhelpers_base.htm')
    template = 'userextensions/custom/manage_service_accounts.html'

    def get(self, request):
        context = dict()
        # get all current service accounts
        context['service_accounts'] = ServiceAccount.objects.filter(group__user=request.user).\
            select_related('user', 'group', 'user__auth_token').order_by('group__name')

        # get list of groups that do not have a service account
        # If the SRV_ACCOUNT_GROUP_FILTER_LIST (list) settings variable is set, only include groups whose name matches a
        # provided regex pattern, otherwise include all groups. Set the SRV_ACCOUNT_ENFORCE_MATCHING (bool) settings
        # variable to False to allow all groups, but maintain naming of the service account based on regex patterns.
        regex_list = getattr(settings, 'SRV_ACCOUNT_GROUP_FILTER_LIST', list())
        if regex_list and getattr(settings, 'SRV_ACCOUNT_ENFORCE_MATCHING', True):
            group_queryset = request.user.groups.none()
            for regex in regex_list:
                group_queryset = group_queryset | request.user.groups.filter(serviceaccount=None, name__regex=regex)
        else:
            group_queryset = request.user.groups.filter(serviceaccount=None)

        context['groups'] = group_queryset.prefetch_related('user_set').order_by('name')
        context['base_template'] = self.base_template
        return render(request, self.template, context=context)



from django.contrib import admin

# import models
from userextensions.models import (Theme, UserFavorite, UserPreference, UserRecent,
                                   ServiceAccount, ServiceAccountTokenHistory)


class ThemeAdmin(admin.ModelAdmin):
    list_display = ("id", "name", "css_file")
    search_fields = ["name", "css_file"]


class UserPreferenceAdmin(admin.ModelAdmin):
    list_display = ("id", "user", "theme", "recents_count", "page_refresh_time", "start_page", "updated_at")
    search_fields = ["host", "user__username"]


class UserRecentAdmin(admin.ModelAdmin):
    list_display = ("id", "user", "url")
    search_fields = ["user__name", "url"]
    list_filter = ["user"]


class UserFavoriteAdmin(admin.ModelAdmin):
    list_display = ("id", "user", "name", "url")
    search_fields = ["name", "user__username", "name", "url"]
    list_filter = ["user"]


class ServiceAccountAdmin(admin.ModelAdmin):
    list_display = ("id", "user", "group", "description", "enabled", "admin_enabled")
    search_fields = ["user__username", "group__name", "description"]
    list_filter = ["enabled"]


class ServiceAccountTokenHistoryAdmin(admin.ModelAdmin):
    list_display = ("id", "actor", "srv_acct", "timestamp")
    search_fields = ["actor__username", "srv_acct__user__username"]
    list_filter = ["actor", "srv_acct"]


# register models
admin.site.register(Theme, ThemeAdmin)
admin.site.register(UserPreference, UserPreferenceAdmin)
admin.site.register(UserFavorite, UserFavoriteAdmin)
admin.site.register(UserRecent, UserRecentAdmin)
admin.site.register(ServiceAccount, ServiceAccountAdmin)
admin.site.register(ServiceAccountTokenHistory, ServiceAccountTokenHistoryAdmin)



from django.db.models.signals import post_save
from django.dispatch import receiver
import sys

# import models
from django.contrib.auth.models import User
from userextensions.models import (UserPreference, UserRecent, ServiceAccount)


@receiver(post_save, sender=User, dispatch_uid='add_user_preference')
def add_user_preference(sender, instance, created, **kwargs):
    """ This post-save signal adds a UserPreference object when a User is created """
    if created:
        UserPreference.objects.create(user=instance)


@receiver(post_save, sender=UserRecent, dispatch_uid='trim_recents')
def trim_recents(sender, instance, created, **kwargs):
    """ This post-save signal trims a users recents to only maintain the x most recent urls, where x is the
    recents_count configured in the UserPreference table """
    # do not execute signal when running tests
    if 'manage.py' in sys.argv[0] and 'test' in sys.argv:
        return

    # get recents count from user preferences if available; else default to 25
    try:
        recents_count = instance.user.preference.recents_count
    except:
        recents_count = 25

    # don't need to trim if recents count is < recents_count
    if UserRecent.objects.filter(user=instance.user).count() <= recents_count:
        return
    recent_id_list = UserRecent.objects.filter(user=instance.user
                                               ).order_by('-updated_at')[:recents_count].values_list('id', flat=True)
    UserRecent.objects.filter(user=instance.user).exclude(pk__in=list(recent_id_list)).delete()


@receiver(post_save, sender=ServiceAccount, dispatch_uid='create_srv_account_token')
def create_srv_account_token(sender, instance, created, **kwargs):
    """ This post-save signal creates a drf token when a new ServiceAccount is created """
    if created:
        instance.create_drf_token()




from django.conf import settings
from django.db import migrations, models
from django.apps import apps
import django.db.models.deletion


def add_user_preferences(apps, schema_editor):
    """ add UserPreference entry for each existing user """
    User = apps.get_model('auth', 'User')
    UserPreference = apps.get_model('userextensions', 'UserPreference')
    for user in User.objects.all():
        UserPreference.objects.create(user=user)


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Theme',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='date/time when this row was first created')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='date/time this row was last updated')),
                ('name', models.CharField(help_text='name of theme', max_length=32, unique=True)),
                ('css_file', models.CharField(blank=True, help_text='path to css file for theme', max_length=32, null=True, unique=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='UserFavorite',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='date/time when this row was first created')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='date/time this row was last updated')),
                ('name', models.CharField(blank=True, help_text='name/label/reference for this favorite', max_length=32, null=True)),
                ('url', models.URLField(help_text='url endpoint')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='favorite', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='UserPreference',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='date/time when this row was first created')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='date/time this row was last updated')),
                ('recents_count', models.IntegerField(blank=True, default=25, help_text='number of recents to keep a record of', null=True)),
                ('page_refresh_time', models.IntegerField(blank=True, default=5, help_text='time, in minutes, to auto-refresh a page (where applicable', null=True)),
                ('theme', models.ForeignKey(blank=True, help_text='theme to use for web pages', null=True, on_delete=django.db.models.deletion.CASCADE, to='userextensions.Theme')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='preference', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='UserRecent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='date/time when this row was first created')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='date/time this row was last updated')),
                ('url', models.URLField(help_text='url endpoint')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='recent', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='userrecent',
            unique_together={('url', 'user')},
        ),
        migrations.AlterUniqueTogether(
            name='userfavorite',
            unique_together={('url', 'user')},
        ),

        migrations.RunPython(add_user_preferences)
    ]
